# aboutbox.py -- about box for qmforge
#
# Copyright (c) Adam Tenderholt, Stanford University, 2005-2007
#                               a-tenderholt@stanford.edu
# Copyright (c) Adam Tenderholt, atenderholt@gmail.com, 2012-2014
#
# This program is free software; you can redistribute it and/or modify  
# it under the terms of the GNU General Public License as published by 
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version with the exceptions noted in the 
# Help dialog.

import sys, os

from PyQt4 import QtCore, QtGui

from ui_about_box import Ui_AboutBox

def get_filename(filename):
    if getattr(sys, 'frozen', False):
        datadir = os.path.dirname(sys.executable)
    else:
        datadir = os.path.dirname("../")

    filename = os.path.join(datadir, filename)
    return filename

class AboutBox(QtGui.QDialog, Ui_AboutBox):
    def __init__(self, parent=None):
        super(AboutBox, self).__init__(parent)

        self.setupUi(self)

        # find path to various text files
        thanks_file = get_filename("THANKS.txt")
        license_file = get_filename("LICENSE.txt")
        changes_file = get_filename("CHANGES.txt")

        # open the files
        with open(thanks_file) as file:
            thanks = file.read()

        with open(license_file) as file:
            license = file.read()

        with open(changes_file) as file:
            changes = file.read()

        # and setup GUI
        self.thanksTextEdit.setPlainText(thanks)
        self.licenseTextEdit.setPlainText(license)
        self.changesTextEdit.setPlainText(changes)

if __name__ == "__main__":
    import sys

    app = QtGui.QApplication(sys.argv)
    window = AboutBox()
    window.show()

    retval = app.exec_()
    sys.exit(retval)
