# fragmentsdialog.py -- dialog for choosing fragment names and files
#
# Copyright (c) Adam Tenderholt, Stanford University, 2005-2007
#                               a-tenderholt@stanford.edu
# Copyright (c) Adam Tenderholt, atenderholt@gmail.com, 2012-2014
#
# This program is free software; you can redistribute it and/or modify  
# it under the terms of the GNU General Public License as published by 
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version with the exceptions noted in the 
# Help dialog.

#created with the help of Qt4 designer and pyuic4

import sys
from PyQt4 import QtCore, QtGui

class FragmentButton(QtGui.QPushButton):
    def __init__(self, parent, lineEdit):
        QtGui.QPushButton.__init__(self, "Browse...", parent)

        self.parent = parent
        self.lineEdit = lineEdit
    
        QtCore.QObject.connect(self, QtCore.SIGNAL("clicked()"), self.slotClicked)

    def slotClicked(self):

        filters = "Log files (*.log *.out *.adfout)\nAll files (*.*)"
        filename = QtGui.QFileDialog.getOpenFileName(self.parent, "Choose fragment file...", "", filters)
        self.lineEdit.setText(filename)
        
class FragmentDialog(QtGui.QDialog):
    def __init__(self, parent = None, name = None):

        QtGui.QDialog.__init__(self)

        self.setObjectName("fragmentDialog")
        self.resize(QtCore.QSize(QtCore.QRect(0,0,680,257).size()).expandedTo(self.minimumSizeHint()))

        self.names = []
        self.paths = []
        self.nameLabels = []
        self.pathLabels = []
        self.buttons = []
        self.layouts = []
        self.oldValue = 2

        self.vboxlayout = QtGui.QVBoxLayout(self)
        self.vboxlayout.setMargin(9)
        self.vboxlayout.setSpacing(6)
        self.vboxlayout.setObjectName("vboxlayout")

        self.hboxlayout = QtGui.QHBoxLayout()
        self.hboxlayout.setMargin(0)
        self.hboxlayout.setSpacing(6)
        self.hboxlayout.setObjectName("hboxlayout")

        self.numLabel = QtGui.QLabel(self)
        self.numLabel.setObjectName("numLabel")
        self.numLabel.setText("Number of Fragments:")
        self.hboxlayout.addWidget(self.numLabel)

        self.spinBox = QtGui.QSpinBox(self)
        self.spinBox.setMinimum(2)
        self.spinBox.setProperty("value",QtCore.QVariant(2))
        self.spinBox.setObjectName("spinBox")
        self.hboxlayout.addWidget(self.spinBox)

        spacerItem2 = QtGui.QSpacerItem(421,20,QtGui.QSizePolicy.Expanding,QtGui.QSizePolicy.Minimum)
        self.hboxlayout.addItem(spacerItem2)
        self.vboxlayout.addLayout(self.hboxlayout)

        spacerItem1 = QtGui.QSpacerItem(20,21,QtGui.QSizePolicy.Minimum,QtGui.QSizePolicy.Minimum)
        self.vboxlayout.addItem(spacerItem1)

        spacerItem = QtGui.QSpacerItem(20,40,QtGui.QSizePolicy.Minimum,QtGui.QSizePolicy.MinimumExpanding)
        self.vboxlayout.addItem(spacerItem)

        self.buttonBox = QtGui.QDialogButtonBox(self)
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(QtGui.QDialogButtonBox.Cancel|QtGui.QDialogButtonBox.NoButton|QtGui.QDialogButtonBox.Ok)
        self.buttonBox.setObjectName("buttonBox")
        self.buttonBox.buttons()[0].setDefault(True)
        self.vboxlayout.addWidget(self.buttonBox)

        self.addFragment()
        self.addFragment()

        QtCore.QObject.connect(self.buttonBox,QtCore.SIGNAL("accepted()"),self.accept)
        QtCore.QObject.connect(self.buttonBox,QtCore.SIGNAL("rejected()"),self.reject)
        QtCore.QObject.connect(self.spinBox, QtCore.SIGNAL("valueChanged(int)"), self.slotNewValue)

    def slotNewValue(self, val):
        if val == self.oldValue:
            return

        if val > self.oldValue:
            for i in range(self.oldValue, val):
                self.addFragment()

            self.oldValue = val

        if val < self.oldValue:
            for i in range(val, self.oldValue):
                self.removeFragment()

            self.oldValue = val

    def removeFragment(self):

        self.vboxlayout.removeItem(self.layouts[-1]) 

        item = self.names.pop()
        item.hide()
        del item
        item = self.paths.pop()
        item.hide()
        del item
        item = self.nameLabels.pop()
        item.hide()
        del item
        item = self.pathLabels.pop()
        item.hide()
        del item
        item = self.buttons.pop()
        item.hide()
        del item
        item = self.layouts.pop()
        
        self.vboxlayout.update()

    def addFragment(self):
        num = len(self.names)

        hboxlayout = QtGui.QHBoxLayout()
        hboxlayout.setMargin(0)
        hboxlayout.setSpacing(6)
        hboxlayout.setObjectName("hboxlayout%i"%num)

        fragLabela = QtGui.QLabel(self)
        fragLabela.setObjectName("fragLabel%ia"%num)
        fragLabela.setText("Fragment Name:")
        hboxlayout.addWidget(fragLabela)

        fragName = QtGui.QLineEdit(self)
        fragName.setText("Frag%i"%(num+1))

        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Policy(0),QtGui.QSizePolicy.Policy(0))
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(fragName.sizePolicy().hasHeightForWidth())
        fragName.setSizePolicy(sizePolicy)
        fragName.setMaximumSize(QtCore.QSize(60,16777215))
        fragName.setObjectName("fragName%i"%num)
        hboxlayout.addWidget(fragName)

        spacerItem = QtGui.QSpacerItem(21,32,QtGui.QSizePolicy.Fixed,QtGui.QSizePolicy.Minimum)
        hboxlayout.addItem(spacerItem)

        fragLabelb = QtGui.QLabel(self)
        fragLabelb.setObjectName("fragLabel%ib"%num)
        fragLabelb.setText("Filename:")
        hboxlayout.addWidget(fragLabelb)

        fragPath = QtGui.QLineEdit(self)
        fragPath.setObjectName("fragPath%i"%num)
        hboxlayout.addWidget(fragPath)

        fragButton = FragmentButton(self, fragPath)
        fragButton.setObjectName("fragButton%i"%num)
        hboxlayout.addWidget(fragButton)
        self.vboxlayout.insertLayout(2+num,hboxlayout)
        
        fragName.show()
        fragPath.show()
        fragLabela.show()
        fragLabelb.show()
        fragButton.show()

        self.names.append(fragName)
        self.paths.append(fragPath)
        self.nameLabels.append(fragLabela)
        self.pathLabels.append(fragLabelb)
        self.buttons.append(fragButton)
        self.layouts.append(hboxlayout)

    def getFragments(self):
        names = []
        paths = []

        for i in range(len(self.names)):
            names.append(str(self.names[i].text()))
            paths.append(str(self.paths[i].text()))

        return names, paths

if __name__ == "__main__":
    app = QtGui.QApplication(sys.argv)
    fragmentDialog = FragmentDialog()
    fragmentDialog.show()
    sys.exit(app.exec_())
