 
# qmforge.py -- code for the mainwindow and the majority of QMForge
#
# Copyright (c) Adam Tenderholt, Stanford University, 2005-2007
#                               a-tenderholt@stanford.edu
# Copyright (c) Adam Tenderholt, atenderholt@gmail.com, 2012-2014
#
# This program is free software; you can redistribute it and/or modify  
# it under the terms of the GNU General Public License as published by 
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version with the exceptions noted in the 
# Help dialog.

#created with the help of Qt designer and pyuic

import sys, os
import logging, cPickle
from os.path import exists
from time import localtime,asctime,time
from math import sqrt, fabs

#import qt modules
from PyQt4 import QtCore,QtGui

#import needed cclib modules
from cclib.parser import ccopen,utils
from cclib.progress import Qt4Progress

from dialogs import *
from fragmentsdialog import *
from aboutbox import AboutBox
from logviewer import *
from customdelegate import *
from formatter import *
from carttosphere import *
from ui_mainwindow import Ui_MainWindow

table = utils.PeriodicTable()

# Tab values
ALL = -1
POPULATION = 0
GEOMETRY = 1
CARTESIAN = 2
FREQUENCIES = 3
TRANSITIONS = 4

METHODS = { "MPA": "Mulliken Population Analysis",
            "LPA": "Lowdin Population Analysis",
            "CSPA": "C-squared Population Analysis",
            "GPA": "Gross Population Analysis" }

class QMForge(QtGui.QMainWindow, Ui_MainWindow):
    def __init__(self,parent = None):
        super(QMForge, self).__init__(parent)
        self.setupUi(self)
        self.tabs = {}
        self.tabs[0] = self.popWidget
        self.tabs[1] = self.geomWidget
        self.tabs[2] = self.cartWidget
        self.tabs[3] = self.freqWidget

#connect slots

#        qtcore.qobject.connect(self.geomwidget, qtcore.signal("signalupdategeometry(int)"), self.cartwidget.slotupdategeometry)
#        qtcore.qobject.connect(self.cartwidget, qtcore.signal("signalupdategeometry(int)"), self.geomwidget.slotupdategeometry)
#        qtcore.qobject.connect(self.tabwidget, qtcore.signal("currentchanged(int)"),self.slottabchanged)


        self.currentGeom = 0

        self.settings = QtCore.QSettings("QMForge_ALT", "QMForge")
        width = self.settings.value("window/width", 1000).toInt()[0]
        height = self.settings.value("window/height", 800).toInt()[0]

        self.resize(width, height)
        self.filename = None

# create lists of actions for easy enable
        self.saveActions = {}
        self.saveActions[ALL] = [self.saveStructureAction, self.saveAnimationAction,
                                    self.saveDataAction, self.saveSpectrumAction ]
        self.saveActions[POPULATION] = []
        self.saveActions[GEOMETRY] = [self.saveStructureAction]
        self.saveActions[CARTESIAN] = [self.saveStructureAction]
        self.saveActions[FREQUENCIES] = [self.saveAnimationAction, self.saveSpectrumAction]
        self.saveActions[TRANSITIONS] = [self.saveSpectrumAction, self.saveDataAction]

        self.populationActions = {}
        self.populationActions[ALL] = [self.MPAAction, self.CSPAAction, self.LPAAction,
                                        self.GPAAction, self.MBOAction, self.OPAAction,
                                        self.menuFragment_Analysis,
                                        self.addFragmentAction, self.removeFragmentAction]
        self.populationActions[POPULATION] = [self.MPAAction, self.CSPAAction, self.LPAAction,
                                                self.GPAAction, self.MBOAction, self.OPAAction,
                                                self.menuFragment_Analysis,
                                                self.addFragmentAction,
                                                self.removeFragmentAction]
        self.populationActions[GEOMETRY] = []
        self.populationActions[CARTESIAN] = []
        self.populationActions[FREQUENCIES] = []
        self.populationActions[TRANSITIONS] = []

        self.viewActions = {}
        self.viewActions[ALL] = [self.centerViewAction, self.orientToXYZAction,
                                    self.orientToYXZAction, self.orientToXZYAction,
                                    self.orientToZXYAction]
        self.viewActions[POPULATION] = [self.centerViewAction, self.orientToXYZAction,
                                            self.orientToYXZAction, self.orientToXZYAction,
                                            self.orientToZXYAction]
        self.viewActions[GEOMETRY] = [self.centerViewAction, self.orientToXYZAction,
                                            self.orientToYXZAction, self.orientToXZYAction,
                                            self.orientToZXYAction]
        self.viewActions[CARTESIAN] = [self.centerViewAction, self.orientToXYZAction,
                                            self.orientToYXZAction, self.orientToXZYAction,
                                            self.orientToZXYAction]
        self.viewActions[FREQUENCIES] = [self.centerViewAction, self.orientToXYZAction,
                                            self.orientToYXZAction, self.orientToXZYAction,
                                            self.orientToZXYAction]
        self.viewActions[TRANSITIONS] = []

        self.modifyActions = {}
        self.modifyActions[ALL] = [self.centerAction, self.rotateToXAction,
                                    self.rotateToYAction, self.rotateToZAction,
                                    self.rotateToXYAction, self.rotateToXZAction,
                                    self.rotateToYZAction]
        self.modifyActions[POPULATION] = []
        self.modifyActions[GEOMETRY] = []
        self.modifyActions[CARTESIAN] = [self.centerAction, self.rotateToXAction,
                                            self.rotateToYAction, self.rotateToZAction,
                                            self.rotateToXYAction, self.rotateToXZAction,
                                            self.rotateToYZAction]
        self.modifyActions[FREQUENCIES] = []
        self.modifyActions[TRANSITIONS] = []

        # needed to make frequency widget update animations sanely
        self.freqWidget.setMainWindowActions(self.saveAnimationAction,
                                                self.saveStructureAction)

        self.reset()


    def fileNew(self):
        print "QMForge.fileNew(): Not implemented yet"

    def reset(self):
        
        #clear results
        self.eigens=[]
        self.results=[]
        self.occs=[]
        self.symms=[]
        self.fragnames=[]
        self.unrestricted=False
        self.filename = None
        
        self.eigensB=[]
        self.resultsB=[]
        self.occsB=[]
        self.symmsB=[]
        
        self.MPAfinished=False
        self.CSPAfinished=False
        self.overlapfinished=False
        
        # Disable all tabs
        self.tabWidget.setTabEnabled(POPULATION, False)
        self.tabWidget.setTabEnabled(GEOMETRY, False)
        self.tabWidget.setTabEnabled(CARTESIAN, False)
        self.tabWidget.setTabEnabled(FREQUENCIES, False)
        self.tabWidget.setTabEnabled(TRANSITIONS, False)

        # Disable menu entries
        for action in self.saveActions[ALL] + self.populationActions[ALL]:
            action.setEnabled(False)
        for action in self.viewActions[ALL] + self.modifyActions[ALL]:
            action.setEnabled(False)

        self.popWidget.reset()
        
    def open(self,filename):
        
        self.reset()

        progress=Qt4Progress("Parsing file...",self)
        progress.setWindowModality(QtCore.Qt.WindowModal)

        viewer = LogViewer(self)
        try:
            parser = ccopen(filename, logging.WARNING)
        except TypeError:
            msg = "The chosen file doesn't appear to be a calculation logfile."
            QtGui.QMessageBox.critical(self, "Error", msg, "Ok")
            return

        parser.logger.addHandler(viewer)
        data = parser.parse(progress)
        progress.update(progress.nstep, "Done")

        self.data=data
        warnings = viewer.ui.textEdit.toPlainText()
        if len(warnings) > 0:
            viewer.show()

#        if hasattr(data, "aonames"):
#            data = self.convertToSpherical(data)

        if hasattr(data,"aonames"):
            self.names = data.aonames
        elif hasattr(data,"fonames"):
            self.names = data.fonames

        if hasattr(self, "names"):
            self.tabWidget.setTabEnabled(POPULATION, True)
            for action in self.populationActions[POPULATION]:
                action.setEnabled(True)
            self.tabWidget.setCurrentIndex(POPULATION)
            self.popWidget.setData(data)

        if hasattr(data,"atomcoords") and hasattr(data,"atomnos"):
            self.tabWidget.setTabEnabled(CARTESIAN, True)
            self.cartWidget.setData(data)

            if len(data.atomcoords) > 1:
                self.geomWidget.setData(data)
                self.tabWidget.setTabEnabled(GEOMETRY, True)
                self.tabWidget.setCurrentIndex(GEOMETRY)

        if hasattr(data, "vibfreqs") and hasattr(data, "vibdisps") and hasattr(data, "atomnos"):
            self.tabWidget.setTabEnabled(FREQUENCIES, True)
            self.tabWidget.setCurrentIndex(FREQUENCIES)
            self.freqWidget.setData(data)

        if hasattr(data, "etenergies") and hasattr(data, "etoscs") and hasattr(data, "etsecs"):
            self.tabWidget.setTabEnabled(TRANSITIONS, True)
            self.tabWidget.setCurrentIndex(TRANSITIONS)
            self.transitionsWidget.setData(data)

        self.data=data
        self.filename=filename

        # now read QSettings, and see if we have names
        names_string = self.settings.value("aonames").toString()
        previous_aonames = []
        if names_string != "":
            previous_aonames = cPickle.loads(str(names_string))
        
        if hasattr(self, "names") and previous_aonames == self.names:
            print "Opening previous file."
        
    def convertToSpherical(self, data):

        ask = True
        starts = []

        for i in range(len(data.aonames)):
            if ask and data.aonames[i].find("XX") > 0:
                retval = QtGui.QMessageBox.question(self, "Cartesian Basis Functions Found", \
                               "The current basis set uses cartesian functions. Do you "+ \
                               "wish to wish to convert them to a spherical basis set?", \
                               QtGui.QMessageBox.Yes, QtGui.QMessageBox.No)

                if retval == QtGui.QMessageBox.Yes:
                    ask = False
                else:
                    return data

            if data.aonames[i].find("XX") > 0:
                starts.append(i)

            data.aonames[i] = data.aonames[i].replace("XX", "S'")
            data.aonames[i] = data.aonames[i].replace("YY", "X2-Y2")
            data.aonames[i] = data.aonames[i].replace("ZZ", "Z2")

        #have we been asked about converting to spherical? if not, then we're
        #already spherical, so just return data
        return data
    
        Dialog = QtGui.QDialog()
        ui = Ui_Dialog()
        ui.setupUi(Dialog)
        Dialog.exec_()

        matrix = numpy.zeros((3,3),"d")
        for i in range(3):
            for j in range(3):
                item = ui.tableWidget.item(i,j)
                val = item.data(QtCore.Qt.DisplayRole).toDouble()[0]
                matrix[i,j] = float(val)

#        matrix = Numeric.zeros((3,3),"d")
#        matrix[:,0] = [1,1,1]
#        matrix[0,1] = sqrt(3/10.0)
#        matrix[1,1] = -1 * sqrt(3/10.0)
#        matrix[0,2] = -0.5 * sqrt(3/5.0)
#        matrix[1,2] = -0.5 * sqrt(3/5.0)
#        matrix[2,2] = 1 * sqrt(3/5.0)

#        print data.mocoeffs[1][35,17]**2, data.mocoeffs[1][35,18]**2, data.mocoeffs[1][35,19]**2
        print reduce(numpy.add, data.mocoeffs[1][35]**2)
        for start in starts:
            for spin in range(len(data.mocoeffs)):
                temp = numpy.inner(data.mocoeffs[spin][:,start:start+3], matrix)
                data.mocoeffs[spin][:,start:start+3] = temp

        print reduce(numpy.add, data.mocoeffs[1][35]**2)
#        print data.mocoeffs[1][35,17]**2, data.mocoeffs[1][35,18]**2, data.mocoeffs[1][35,19]**2

#        for i in range(len(data.mocoeffs[1])):
#            print i, reduce(Numeric.add, data.mocoeffs[1][i]**2)

        return data

    def fileOpen(self):
       
        filters = "Log files (*.log *.out *.adfout)\nAll files (*.*)"
        if not self.filename:
            dirpath = QtCore.QDir.homePath()
        else:
            info = QtCore.QFileInfo(self.filename)
            dirpath = info.absoluteFilePath()

        filename=QtGui.QFileDialog.getOpenFileName(self, "Open Log File...", dirpath, filters)
        if not (filename):
            return
        
        self.open(str(filename))
        
    def saveStructure(self):
        namelist = ["XYZ Coordinates", "Protein Data Bank", "Z-matrix", "Gaussian Input"]
        extlist = ["xyz", "pdb", "zmat", "com"]
        obformatlist = ["xyz", "pdb"," fh", "com"]
        typelist = ["XYZ", "PDB", "ZMat", "G03"]

        formatdialog = SaveFormatDialog(self, namelist)
        result = formatdialog.exec_()
        if result == QtGui.QDialog.Rejected:
            return

        index = formatdialog.comboBox.currentIndex()
        filters = "%s (*.%s) ;; *" % (namelist[index], extlist[index])

        if self.tabWidget.currentIndex() == FREQUENCIES:
            wavenumber = self.freqWidget.getCurrentFrequency()
            disp = self.freqWidget.glWidget.current

            if disp > 0:
                temp = "_%i_plus%i" % (wavenumber, disp)
            else:
                temp = "_%i_minus%i" % (wavenumber, fabs(disp))

            suggest = os.path.splitext(self.filename)[0] + temp + "." + extlist[index]
        else:
            suggest = os.path.splitext(self.filename)[0] + "." + extlist[index]

        filename = QtGui.QFileDialog.getSaveFileName(self, "Save %s file"%typelist[index], \
                                suggest, filters)

        if not filename:
            return

        current = self.tabWidget.currentWidget()
        current.glWidget.saveFile(filename, obformatlist[index])

    def saveData(self):
        self.transitionsWidget.saveData(self.filename)

    def saveSpectrum(self):
        self.tabWidget.currentWidget().saveSpectrum(self.filename)

    def saveAnimation(self):

        wavenumber = self.freqWidget.getCurrentFrequency()

        # build some useful strings
        if wavenumber < 0:
            suggest = os.path.splitext(self.filename)[0] + "_%ii.gif" % fabs(wavenumber)
        else:
            suggest = os.path.splitext(self.filename)[0] + "_%i.gif" % wavenumber
        filters = "GIF (*.gif)"

        filename = QtGui.QFileDialog.getSaveFileName(self, "Save animation...", suggest, filters)
        if not filename:
            return

        self.freqWidget.saveAnimation(filename)

    def filePrint(self):
        print "QMForge.filePrint(): Not implemented yet"

    def fileExit(self):
        print "QMForge.fileExit(): Not implemented yet"

    def fragAdd(self):
        self.fragView.addFragment()
        
    def slotUpdateGeom(self,value):
        self.geomWidget.setMolecule(self.data.atomcoords[value],self.data.atomnos)

    def slotUpdateCart(self,value):
        self.cartWidget.setMolecule(self.data.atomcoords[value],self.data.atomnos)
        
        self.cartModel.clear()
        self.cartModel.setHorizontalHeaderLabels(["Element", "x", "y", "z"])
        self.cartView.setColumnWidth(0,55)
        self.cartView.setColumnWidth(1,75)
        self.cartView.setColumnWidth(2,75)
        self.cartView.setColumnWidth(3,75)

        data = self.data
        for i in range(len(data.atomnos)):
                elementItem = QtGui.QStandardItem(table.element[data.atomnos[i]])
                xItem = QtGui.QStandardItem(str(data.atomcoords[value][i][0]))
                yItem = QtGui.QStandardItem(str(data.atomcoords[value][i][1]))
                zItem = QtGui.QStandardItem(str(data.atomcoords[value][i][2]))
                self.cartModel.appendRow([elementItem,xItem,yItem,zItem])

    def slotUpdateCartData(self,topLeft,bottomRight):
        row = topLeft.row()
        col = topLeft.column()

        if row != bottomRight.row() or col != bottomRight.column():
            print "There's an error with editing Cartesian data"
            return

        item = self.cartModel.item(row,col)
        if col == 0:
            tempdata = item.data(QtCore.Qt.DisplayRole).toString()
            data = table.number[str(tempdata)]

        else:
            data, ok = item.data(QtCore.Qt.DisplayRole).toDouble()

        self.cartWidget.updateMolecule(row,col,data)

    def changeCartSelection(self, current, previous):
        indexes = self.selectionModel.selectedIndexes()
        dict = {}
        for index in indexes:
            dict[index.row()] = 1
        self.cartWidget.setSelected(dict.keys())

    def slotSaveFragments(self):
#save fragment list as a text file for easy processing
        try:
            import simplejson
        except ImportError:
            print "simplejson not installed, returning..."
            return

        fragmodel = self.popWidget.fragmodel
        fragnames,fraglists=fragmodel.getFragments()

        for i in range(len(fragnames)):
            fragnames[i] = str(fragnames[i])

        file = open("fraglists.dat","w")
        simplejson.dump(fragnames, file)
        file.write("\n")
        simplejson.dump(fraglists, file)
        file.close()

    def calcMPA(self):
        self.doPopulationAnalysis("MPA")

    def calcCSPA(self):
        self.doPopulationAnalysis("CSPA")

    def calcLPA(self):
        self.doPopulationAnalysis("LPA")

    def calcGPA(self):
        from cclib.method import MPA, LPA

        if not self.checkMOs("GPA"):
            return
        if not self.checkOverlaps("GPA"):
            return

        progress = Qt4Progress("Calculating...",self)
        progress.setWindowModality(QtCore.Qt.WindowModal)

        mpa = MPA(self.data, progress, logging.ERROR)
        mpa.calculate()

        lpa = LPA(self.data, progress, logging.ERROR)
        lpa.calculate()

        formatter = Formatter(self.filename)
        displayText = formatter.printGrossPopulation(self.data, mpa, lpa)
        self.displayResults(displayText, ".txt")

    def checkMOs(self, method):
        full_method = METHODS[method]
        mesg = "The logfile is missing information about molecule orbital energies "
        mesg += "and/or coefficients; therefore, %s is not available." % full_method

        if not hasattr(self.data, "moenergies") or not hasattr(self.data, "mocoeffs"):
            QtGui.QMessageBox.critical(self, "Error", QtCore.QString(mesg), "Ok")
            return False

        return True

    def checkOverlaps(self, method):
        full_method = METHODS[method]
        mesg = "The logfile is missing orbital overlap information; therefore, "
        mesg += "%s is not available." % full_method

        if not hasattr(self.data, "aooverlaps") or hasattr(self.data, "fooverlaps"):
            QtGui.QMessageBox.critical(self,"Error",QtCore.QString(mesg), "Ok")
            return False

        return True

    def doPopulationAnalysis(self, method):
#calculate population analysis using cclib and print results using QMForge functions

        fragmodel = self.popWidget.fragmodel
        nOver = method == "LPA" or method == "MPA"
        
        if not self.checkMOs(method):
            return
        if nOver and not self.checkOverlaps(method):
            return

        fragnames, fraglists = fragmodel.getFragments()

        if method == "MPA":
            from cclib.method import MPA as Method
        elif method == "CSPA":
            from cclib.method import CSPA as Method
        elif method == "LPA":
            from cclib.method import LPA as Method
        else:
            mesg = "Unsupported method type."
            QtGui.QMessageBox.critical(self,"Error",QtCore.QString(mesg), "Ok")
            return

        progress = Qt4Progress("Calculating...",self)
        progress.setWindowModality(QtCore.Qt.WindowModal)
        
        m = Method(self.data,progress,logging.ERROR)
        m.calculate(fraglists)

        formatter = Formatter(self.filename)
        displayText = formatter.printAllPA(self.data, fraglists, fragnames, m, method)
        self.displayResults(displayText, ".txt")
        
        if method == "MPA":
            self.MPAfinished = True
            self.CSPAfinished = False
        elif method == "CSPA":
            self.MPAfinished = False
            self.CSPAfinished = True
        
    def calcFragMPA(self):
#analyze MO's in terms of fragment MOs

        if not hasattr(self, "fragmentdialog"):
            self.fragmentdialog = FragmentDialog(self)
        retval = self.fragmentdialog.exec_()
        if not retval:
            return

        names, paths = self.fragmentdialog.getFragments()
        datas = []
        progress=Qt4Progress("Parsing...",self)
        viewer = LogViewer(self)
        
        for i in range(len(paths)):
            parser = ccopen(paths[i], logging.WARNING)
            parser.logger.addHandler(viewer)
            data = parser.parse(progress)
            progress.update(progress.nstep, "Done")

            data.name = names[i]
            datas.append(data)

        from cclib.method import MPA, FragmentAnalysis
        method = FragmentAnalysis(self.data, None, logging.ERROR)
        method.logger.addHandler(viewer)
        retval = method.calculate(datas)
    
        if retval:

            progress=Qt4Progress("Calculating...",self)

            m=MPA(method,progress,logging.ERROR)
            m.calculate()

            formatter = Formatter(self.filename)
            displayText=formatter.printExtendedPA(self.data, method,m,"Extended MPA")
            self.displayResults(displayText,".txt")

        warnings = viewer.ui.textEdit.toPlainText()

        if len(warnings) > 0:
            viewer.show()

    def calcFragCSPA(self):
#analyze MO's in terms of fragment MOs

        if not hasattr(self, "fragmentdialog"):
            self.fragmentdialog = FragmentDialog(self)
        retval = self.fragmentdialog.exec_()
        if not retval:
            return

        names, paths = self.fragmentdialog.getFragments()
        datas = []
        progress=Qt4Progress("Parsing...",self)
        viewer = LogViewer(self)

        for i in range(len(paths)):
            parser = ccopen(paths[i], logging.WARNING)
            parser.logger.addHandler(viewer)
            data = parser.parse(progress)
            progress.update(progress.nstep, "Done")

            data.name = names[i]
            datas.append(data)

        from cclib.method import CSPA, FragmentAnalysis
        method = FragmentAnalysis(self.data, None, logging.WARNING)
        method.logger.addHandler(viewer)
        retval = method.calculate(datas)
        
        if retval:
            progress=Qt4Progress("Calculating...",self)

            m=CSPA(method,progress,logging.ERROR)
            m.calculate()

            formatter = Formatter(self.filename)
            displayText=formatter.printExtendedPA(self.data, method,m,"Extended CSPA")
            self.displayResults(displayText,".txt")

        warnings = viewer.ui.textEdit.toPlainText()

        if len(warnings) > 0:
            viewer.show()

    def calcFragCDA(self):
#charge decomposition analysis

        self.fragmentdialog = FragmentDialog(self)
        self.fragmentdialog.spinBox.setEnabled(False)
        retval = self.fragmentdialog.exec_()
        if not retval:
            return

        names, paths = self.fragmentdialog.getFragments()
        datas = []
        progress=Qt4Progress("Parsing...",self)
        viewer = LogViewer(self)

        for i in range(len(paths)):
            parser = ccopen(paths[i], logging.WARNING)
            parser.logger.addHandler(viewer)
            data = parser.parse(progress)
            progress.update(progress.nstep, "Done")

            data.name = names[i]
            datas.append(data)

        from cclib.method import CDA
        
        method = CDA(self.data, progress, logging.WARNING)
        method.logger.addHandler(viewer)
        retval = method.calculate(datas)

        if retval:

            formatter = Formatter(self.filename)
            displayText = formatter.printCDA(self.data, names, paths, method)
            self.displayResults(displayText, ".cda")

        warnings = viewer.ui.textEdit.toPlainText()

        if len(warnings) > 0:
            viewer.show()

    def calcOPA(self):
#calculate the overlap populatino

#        aomodel = self.popWidget.aomodel
        fragmodel = self.popWidget.fragmodel

#        if aomodel.rowCount() != 0:
#            QtGui.QMessageBox.critical(self,"Error",QtCore.QString("Please assign all orbitals to a fragment before trying an Overlap Population Analysis."),"Ok")
#            return

        fragnames,fraglists=fragmodel.getFragments()

        if not ( hasattr(self.data,"aooverlaps") or hasattr(self.data,"fooverlaps") ):
            mesg = "The logfile is missing orbital overlap information; therefore,"
            mesg += " Overlap Population Analysis is not available."

            QtGui.QMessageBox.critical(self,"Error",QtCore.QString(mesg), "Ok")
            return

        from cclib.method import OPA

        progress = Qt4Progress("Calculating...",self)
        method = OPA(self.data, progress, logging.ERROR)
        method.calculate(fraglists)

        formatter = Formatter(self.filename)
        displayText = formatter.printAllOP(self.data, fraglists, fragnames, method, "OPA")
        self.displayResults(displayText,".opa")
   
    def calcMBO(self):
#calculate Mayer Bond Orders

        if not ( hasattr(self.data,"aooverlaps") or hasattr(self.data,"fooverlaps") ):
            mesg = "The logfile is missing orbital overlap information; therefore,"
            mesg += " Mayer bond orders are not available."
            QtGui.QMessageBox.critical(self,"Error",QtCore.QString(mesg), "Ok")
            return
      
        from cclib.method import MBO
        from cclib.parser.utils import PeriodicTable

        progress=Qt4Progress("Calculating...",self)
        method = MBO(self.data,progress,logging.ERROR)
        method.calculate()
        isRestricted = ( len(method.fragresults) == 1 )

        displayText="Mayer bond order results of %s\n" % self.filename
        displayText+="Created on %s\n\n"%asctime()
        
#make list of atoms
        atoms=[]
        atomname,orbname = self.names[0].split('_')
        atoms.append(atomname)
        for i in range(len(self.names)):
            atomname,orbname = self.names[i].split('_')
            
            if atomname != atoms[-1]:
                atoms.append(atomname)
            
#write out total MBO
        if isRestricted:
            displayText+="Total\n"
        else:
            displayText+="Total (alpha+beta contributions)\n"
        displayText+="================================\n\n"
        
        base=0
        current=0
        natoms = len(atoms)
        
        while(base<natoms):
        
          displayText+="Total  "
          if natoms-base>=10:
            ncols=10
          else:
            ncols=natoms-base
            
          for i in range(ncols):
            displayText+=" %5s "%(atoms[base+i])
          displayText+="\n"
          displayText+="-------"
          for i in range(ncols):
            displayText+="-------"
          displayText+="\n"
          
          for i in range(base,natoms):
          
            displayText+="%-6s "%(atoms[i])
            for j in range(ncols):
                if isRestricted:
                    displayText+=" %6.3f"%(method.fragresults[0,base+j,i])
                else:
                    displayText+=" %6.3f"%(method.fragresults[0,base+j,i] \
                                            + method.fragresults[1,base+j,i])
                    
            displayText+="\n"
            
          base+=ncols
          displayText+="\n"
    

        if isRestricted:
            
            self.displayResults(displayText,".mbo")
            return

#write out alpha MBO
        displayText+="\n"
        displayText+="Alpha contributions\n"
        displayText+="===================\n\n"
        
        base=0
        current=0
        
        while(base<natoms):
        
          displayText+="Alpha  "
          if natoms-base>=10:
            ncols=10
          else:
            ncols=natoms-base
            
          for i in range(ncols):
            displayText+=" %5s "%(atoms[base+i])
          displayText+="\n"
          displayText+="-------"
          for i in range(ncols):
            displayText+="-------"
          displayText+="\n"
          
          for i in range(base,natoms):
          
            displayText+="%-6s "%(atoms[i])
            for j in range(ncols):
              displayText+=" %6.3f"%(method.fragresults[0,base+j,i])
            displayText+="\n"
            
          base+=ncols
          displayText+="\n"
        
#write out beta MBO
        displayText+="\n"
        displayText+="Beta contributions\n"
        displayText+="==================\n\n"
        
        base=0
        current=0
        
        while(base<natoms):
        
          displayText+="Beta   "
          if natoms-base>=10:
            ncols=10
          else:
            ncols=natoms-base
            
          for i in range(ncols):
            displayText+=" %5s "%(atoms[base+i])
          displayText+="\n"
          displayText+="-------"
          for i in range(ncols):
            displayText+="-------"
          displayText+="\n"
          
          for i in range(base,natoms):
          
            displayText+="%-6s "%(atoms[i])
            for j in range(ncols):
              displayText+=" %6.3f"%(method.fragresults[1,base+j,i])
            displayText+="\n"
            
          base+=ncols
          displayText+="\n"

        self.displayResults(displayText,".mbo")
          
    def displayResults(self,text,suffix):
    
        Dialog = QtGui.QDialog()
        ui = Results_Dialog()
        ui.setupUi(Dialog)
        
        ui.textEdit.setFontFamily("Courier")
        ui.textEdit.setPlainText(text)
        
        Dialog.resize(800,600)
        value=Dialog.exec_()

        if(value):
            
            suggest=self.filename.split(".")[0]+suffix
            qstr=QtGui.QFileDialog.getSaveFileName(self,QtCore.QString("Save results..."),QtCore.QString(suggest),QtCore.QString("*"+suffix+" ;; *"))

            if not qstr:
                return
            
            file=open(qstr,"w")
            file.write(text)
            file.close()
            
    def calcDOS(self):

        lumoindex=self.occs.index(0)
        homoindex=lumoindex-1
        min=self.eigens[homoindex]-10
        max=self.eigens[lumoindex]+10
        
        dos=DOSDialog(self.filename[:-4],".dos",(self.results,self.eigens),self.unrestricted,(self.resultsB,self.eigensB),self.fragnames,min,max)
        dos.exec_loop()
        
    def calcODOS(self):
        
        lumoindex=self.occs.index(0)
        homoindex=lumoindex-1
        min=self.eigens[homoindex]-10
        max=self.eigens[lumoindex]+10
        
        names=[]
        
        for i in range(len(self.fragnames)-1):
            
            for j in range(i+1,len(self.fragnames)):
                
                names.append(self.fragnames[i]+" & "+self.fragnames[j])
                
        dos=DOSDialog(self.filename[:-4],".opd",(self.Oresults,self.eigens),self.unrestricted,(self.OresultsB,self.eigensB),names,min,max,False)
        dos.exec_loop()
        
    #------------------------------------begin calcOverlap-------------------------------------------------
    def calcOverlap(self):
        
        self.frags,self.fragnames=self.fragView.getFragments()
        if(self.overlapfinished):
            displayText=self.printAllOP((self.eigens,self.symms,self.occs,self.Oresults),(self.eigensB,self.symmsB,self.occsB,self.OresultsB),"Overlap Population Analysis")
            self.displayResults(displayText,".opt")
            return
        
        coeffsA,coeffsB=self.parser.getCoeffs()

        #remove empty fragments
        i=0
        while(i<len(self.frags)):
            if(len(self.frags[i])==0):
                self.frags.pop(i)
                self.fragnames.pop(i)
            else:
                i+=1
                
        occsA,occsB=self.parser.getOccs()
        eigensA,eigensB=self.parser.getEigens()
        symmsA,symmsB=self.parser.getSymms()
        
        overlap=self.parser.getOverlap()
        if(len(overlap)==0):
            return
        
        resultsA=[]
        resultsB=[]
        
        sizeA=len(coeffsA)
        sizeB=len(coeffsB)
        steps=sizeA+sizeB
        
        pd=QProgressDialog("Calculating overlap for MO# 0...","Cancel",steps,self,"progress",True)
        pd.setMinimumDuration(1)
        
        pd.setCaption("Overlap Population")
        
        for i in range(sizeA):
            
            pd.setProgress(i)
            if(pd.wasCancelled()):
                return
            
            pd.setLabelText(QString("Calculating overlap for MO# %i..."%(i+1,)))
            
            tempResults=op.results(self.frags,coeffsA[i],overlap)
            resultsA.append(tempResults)
            
        for i in range(sizeB):
            
            pd.setProgress(i+sizeA)
            if(pd.wasCancelled()):
                return
            
            pd.setLabelText(QString("Calculating overlap for beta MO# %i..."%(i+1,)))
            
            tempResults=op.results(self.frags,coeffsB[i],overlap)
            resultsB.append(tempResults)
                
        pd.setProgress(steps)
            
        resultString=""
        resultString=self.printAllOP((eigensA,symmsA,occsA,resultsA),(eigensB,symmsB,occsB,resultsB),"Overlap Population Analysis")
        
        self.displayResults(resultString,".opt")
        
        self.Oresults=resultsA
        self.OresultsB=resultsB
        
        self.eigens=eigensA
        self.eigensB=eigensB
        
        self.symms=symmsA
        self.symmsB=symmsB
        
        self.occs=occsA
        self.occsB=occsB
        
        self.overlapfinished=True
        self.calcODOSAction.setEnabled(True)
        
    #----------------------------end calcOverlap---------------------------------------------------------
    def centerMolecule(self):
        self.cartWidget.centerMolecule()

    def rotateToX(self):
        self.cartWidget.rotateToX()

    def rotateToY(self):
        self.cartWidget.rotateToY()

    def rotateToZ(self):
        self.cartWidget.rotateToZ()

    def rotateToXY(self):
        self.cartWidget.rotateToXY()

    def rotateToXZ(self):
        self.cartWidget.rotateToXZ()

    def rotateToYZ(self):
        self.cartWidget.rotateToYZ()

    def centerView(self):
        index = self.tabWidget.currentIndex()
        self.tabs[index].glWidget.center()

    def orientXYZ(self):
        index = self.tabWidget.currentIndex()
        self.tabs[index].glWidget.orient(0, 0, 0)

    def orientYXZ(self):
        index = self.tabWidget.currentIndex()
        self.tabs[index].glWidget.orient(0, 0, -90)

    def orientXZY(self):
        index = self.tabWidget.currentIndex()
        self.tabs[index].glWidget.orient(-90, 0, 0)

    def orientZXY(self):
        index = self.tabWidget.currentIndex()
        self.tabs[index].glWidget.orient(90, 90, 180)

    def helpIndex(self):
        
        QMessageBox.information( self, "Help", "Help documents have not been created yet.\nPlease email the author (atenderholt at users dot sourceforge dot net) with any questions or comments.", QMessageBox.Ok)

    def helpContents(self):
        print "QMForge.helpContents(): Not implemented yet"

    def helpAbout(self):
        dialog = AboutBox()
        dialog.exec_()

    def addSlot(self):
        if self.tabWidget.currentIndex() == 0:
            self.popWidget.fragmodel.addFragment()
        elif self.tabWidget.currentIndex() == 2:
            self.cartWidget.addAtom()

    def removeSlot(self):
        if self.tabWidget.currentIndex() == 0:
            self.popWidget.removeFragments()
        elif self.tabWidget.currentIndex() == 2:
            self.cartWidget.removeAtom()

    def quitSlot(self):
        print "Preparing for quit"

        if hasattr(self, "names"):
            pickle_string = cPickle.dumps(self.names)
            self.settings.setValue("aonames", pickle_string)
            self.settings.sync()

        QtGui.qApp.quit()

    def closeEvent(self, event):
        self.quitSlot()

    def __tr(self,s,c = None):
        return qApp.translate("QMForge",s,c)

    def slotTabChanged(self,value):
        """
        POPULATION = 0
        GEOMETRY = 1
        CARTESIAN = 2
        FREQUENCIES = 3
        TRANSITIONS = 4
        """

        self.freqWidget.endAnimation()
# disable all actions
        for action in self.saveActions[ALL] + self.populationActions[ALL]:
            action.setEnabled(False)
        for action in self.viewActions[ALL] + self.modifyActions[ALL]:
            action.setEnabled(False)

# now enable for appropriate type
        for action in self.saveActions[value] + self.populationActions[value]:
            action.setEnabled(True)
        for action in self.viewActions[value] + self.modifyActions[value]:
            action.setEnabled(True)

        if value == GEOMETRY:
            num = self.cartWidget.spinBox.value()
            self.geomWidget.slotUpdateGeometry(num)

        if value == CARTESIAN:
            num = self.geomWidget.spinBox.value()
            self.cartWidget.slotUpdateGeometry(num)

        if value == FREQUENCIES:
            self.freqWidget.beginAnimation()

    def resizeEvent(self, event):
        size = event.size()

        self.settings.setValue("window/width", size.width())
        self.settings.setValue("window/height", size.height())

if __name__=='__main__':
    app = QtGui.QApplication(sys.argv)
    window = QMForge()
    window.show()
    if len(sys.argv)==2 and exists(sys.argv[1]):
        window.open(sys.argv[1])
        
    result = app.exec_()
    del window
    del app
    sys.exit(result)

